Package: NoRCE
Type: Package
Title: NoRCE: Noncoding RNA Sets Cis Annotation and Enrichment
Version: 1.25.0
Authors@R: c(person("Gulden", "Olgun", 
           email = "gulden@cs.bilkent.edu.tr", 
		   role = c("aut", "cre")))
Description: While some non-coding RNAs (ncRNAs) are assigned critical
        regulatory roles, most remain functionally uncharacterized.
        This presents a challenge whenever an interesting set of ncRNAs
        needs to be analyzed in a functional context. Transcripts
        located close-by on the genome are often regulated together.
        This genomic proximity on the sequence can hint to a functional
        association. We present a tool, NoRCE, that performs cis
        enrichment analysis for a given set of ncRNAs. Enrichment is
        carried out using the functional annotations of the coding
        genes located proximal to the input ncRNAs. Other biologically
        relevant information such as topologically associating domain
        (TAD) boundaries, co-expression patterns, and miRNA target
        prediction information can be incorporated to conduct a richer
        enrichment analysis. To this end, NoRCE includes several
        relevant datasets as part of its data repository, including
        cell-line specific TAD boundaries, functional gene sets, and
        expression data for coding & ncRNAs specific to cancer.
        Additionally, the users can utilize custom data files in their
        investigation. Enrichment results can be retrieved in a tabular
        format or visualized in several different ways. NoRCE is
        currently available for the following species: human, mouse,
        rat, zebrafish, fruit fly, worm, and yeast.
License: MIT + file LICENSE
Depends: R (>= 4.4.0)
Imports:
        KEGGREST,png,dplyr,graphics,RSQLite,DBI,tidyr,grDevices,stringr,Seqinfo,
        S4Vectors,SummarizedExperiment,reactome.db,rWikiPathways,RCurl,
        dbplyr,utils,ggplot2,igraph,stats,reshape2,readr, GO.db,
        biomaRt,rtracklayer,IRanges,GenomicRanges,GenomicFeatures,AnnotationDbi,methods
Encoding: UTF-8
RoxygenNote: 7.3.2
Suggests: knitr,
        TxDb.Hsapiens.UCSC.hg38.knownGene,TxDb.Drerio.UCSC.danRer10.refGene,
        TxDb.Mmusculus.UCSC.mm10.knownGene,TxDb.Dmelanogaster.UCSC.dm6.ensGene,
        testthat,TxDb.Celegans.UCSC.ce11.refGene,rmarkdown,
        TxDb.Rnorvegicus.UCSC.rn6.refGene,TxDb.Hsapiens.UCSC.hg19.knownGene,
        org.Mm.eg.db,
        org.Rn.eg.db,org.Hs.eg.db,org.Dr.eg.db,BiocGenerics,
        org.Sc.sgd.db, org.Ce.eg.db,org.Dm.eg.db, markdown
VignetteBuilder: knitr
biocViews: BiologicalQuestion, DifferentialExpression,
        GenomeAnnotation, GeneSetEnrichment, GeneTarget,
        GenomeAssembly, GO
LazyData: true
BugReports: https://github.com/guldenolgun/NoRCE/issues
Config/pak/sysreqs: libglpk-dev make libbz2-dev libicu-dev liblzma-dev
        libpng-dev libxml2-dev libssl-dev libx11-dev xz-utils
        zlib1g-dev
Repository: https://bioc.r-universe.dev
Date/Publication: 2026-04-28 12:52:07 UTC
RemoteUrl: https://github.com/bioc/NoRCE
RemoteRef: HEAD
RemoteSha: 6b521031de2dc83b48cf5e667f01c02c33b7c8d7
NeedsCompilation: no
Packaged: 2026-05-30 07:58:57 UTC; root
Author: Gulden Olgun [aut, cre]
Maintainer: Gulden Olgun <gulden@cs.bilkent.edu.tr>
Built: R 4.6.0; ; 2026-05-30 11:35:13 UTC; unix
