# Generated by roxygen2: do not edit by hand

export(KeggEnrichment)
export(WikiEnrichment)
export(assembly)
export(calculateCorr)
export(convertGMT)
export(convertGeneID)
export(corrbased)
export(corrbasedMrna)
export(createNetwork)
export(drawDotPlot)
export(extractBiotype)
export(filterBiotype)
export(geneGOEnricher)
export(genePathwayEnricher)
export(geneRegionGOEnricher)
export(geneRegionPathwayEnricher)
export(getGoDag)
export(getKeggDiagram)
export(getNearToExon)
export(getNearToIntron)
export(getReactomeDiagram)
export(getTADOverlap)
export(getUCSC)
export(getmiRNACount)
export(goEnrichment)
export(listTAD)
export(mirnaGOEnricher)
export(mirnaPathwayEnricher)
export(mirnaRegionGOEnricher)
export(mirnaRegionPathwayEnricher)
export(predictmiTargets)
export(reactomeEnrichment)
export(setParameters)
export(topEnrichment)
export(writeEnrichment)
exportClasses(NoRCE)
import(GO.db)
import(GenomicFeatures)
import(dbplyr)
import(readr)
import(stringr)
importFrom(AnnotationDbi,Term)
importFrom(AnnotationDbi,mappedkeys)
importFrom(GenomicFeatures,as.list)
importFrom(GenomicFeatures,intronsByTranscript)
importFrom(GenomicRanges,GRanges)
importFrom(GenomicRanges,as.data.frame)
importFrom(GenomicRanges,duplicated)
importFrom(GenomicRanges,end)
importFrom(GenomicRanges,findOverlaps)
importFrom(GenomicRanges,intersect)
importFrom(GenomicRanges,match)
importFrom(GenomicRanges,merge)
importFrom(GenomicRanges,order)
importFrom(GenomicRanges,pintersect)
importFrom(GenomicRanges,resize)
importFrom(GenomicRanges,split)
importFrom(GenomicRanges,start)
importFrom(GenomicRanges,strand)
importFrom(GenomicRanges,width)
importFrom(IRanges,IRanges)
importFrom(IRanges,subsetByOverlaps)
importFrom(Seqinfo,seqlevels)
importFrom(biomaRt,getBM)
importFrom(biomaRt,useEnsembl)
importFrom(biomaRt,useMart)
importFrom(dplyr,"%>%")
importFrom(ggplot2,aes)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(igraph,"E<-")
importFrom(igraph,"V<-")
importFrom(igraph,E)
importFrom(igraph,V)
importFrom(igraph,cluster_optimal)
importFrom(igraph,degree)
importFrom(igraph,graph_from_data_frame)
importFrom(igraph,layout_with_fr)
importFrom(igraph,norm_coords)
importFrom(reshape2,melt)
importFrom(rtracklayer,"genome<-")
importFrom(rtracklayer,browserSession)
importFrom(rtracklayer,genome)
importFrom(rtracklayer,getTable)
importFrom(rtracklayer,import)
importFrom(rtracklayer,ucscTableQuery)
importFrom(stats,chisq.test)
importFrom(stats,cor)
importFrom(stats,cor.test)
importFrom(stats,fisher.test)
importFrom(stats,na.omit)
importFrom(stats,p.adjust)
importFrom(stats,pbinom)
importFrom(stats,phyper)
importFrom(stats,reorder)
importFrom(stats,setNames)
importFrom(stats,var)
importFrom(utils,browseURL)
importFrom(utils,installed.packages)
importFrom(utils,read.table)
importFrom(utils,write.table)
