# NoRCE: Non-coding RNA Sets Cis Enrichment Tool

Non-coding RNA Sets Cis Enrichment Tool (NoRCE) performs cis enrichment analysis for a given set of ncRNAs. Enrichment is carried out by using the functional annotations of the coding genes located proximally to the input ncRNAs. NoRCE allows incorporating other biological information such as the topologically associating domain (TAD) regions, co-expression patterns, and miRNA target information. NoRCE repository includes several data files, such as cell line specific TAD regions, functional gene sets, and cancer expression data. Additionally, users can input custom data files. Results can be retrieved in a tabular format or viewed as graphs. NoRCE is currently available for the following species: human, mouse, rat, zebrafish, fruit fly, worm and yeast. NoRCE accepts gene lists and gene regions as an input. 


Tutorial for how to run this package and example codes can be found at https://rpubs.com/golgun/649085
